using System;
using System.Collections.Generic;
using System.Text;

namespace Environment
{
    public class Grid<T> where T :ICell,  new()
    {
        #region Fields
        int _width;
        int _height;
        T[][] _cells;
        List<T> _linearCellList;
        #endregion

        #region Properties
        public  T[][] Cells
        {
            get { return _cells; }
            set { _cells = value; }
        }
        public int Width
        {
            get { return _width; }
            set { _width = value; }
        }
        public int Height
        {
            get { return _height; }
            set { _height = value; }
        }
        public List<T> LinearCellList
        {
            get { return _linearCellList; }
            set { _linearCellList = value; }
        }
        #endregion

        #region Constructors
        public Grid(int width, int height)
        {
            _width = width;
            _height = height;
            _linearCellList = new List<T>();

            _cells = new  T[height][];
            for (int i = 0; i < _height; i++)
            {
                _cells[i] = new  T[width];
                for (int j = 0; j < _width; j++)
                {
                    T cell=new T();
                    _cells[i][j] = cell;
                    _linearCellList.Add(cell);
                }
            }


            //Set neighbours
            for (int i = 0; i < _height; i++)
            {
                for (int j = 0; j < _width; j++)
                {
                    for (int u = -1; u <= 1; u++)
                    {
                        for (int v = -1; v <= 1; v++)
                        {
                            int x = j + u;
                            int y = i + v;

                            if (y >= 0 && y < Height && x >= 0 && x < Width)
                            {
                                if (!(u == v && v == 0 && u == 0))
                                {
                                    _cells[i][j].Add_Neighbour(_cells[y][x]);
                                }
                            }
                            else
                            {
                                int newY = y;
                                int newX = x;
                                if (y < 0 )
                                {
                                    newY = _height-1;
                                }
                                if (x< 0)
                                {
                                    newX = _width-1;
                                }
                                if (y >= _height)
                                {
                                    newY = 0;
                                }
                                if (x >= _width)
                                {
                                    newX = 0;
                                }


                                _cells[i][j].Add_Neighbour(_cells[newY][newX]);
                            }
                        }
                    }
                }
            }
        }
        #endregion

        #region Accessors
        public  T this[int i, int j]
        {
            get
            {
                if (i >= 0 && j >= 0 && j < _width && i < _height)
                {
                    return _cells[i][j];
                }
                return default(T);

            }
            set { _cells[i][j] = value; }
        }
        #endregion
    }
}
